// Alipay
package main

import (
	"crypto/md5"
	"crypto/tls"
	"encoding/hex"
	"fmt"
	"io"
	"io/ioutil"
	"log"
	"math/rand"

	"net/http"
	"net/url"
	"os"
	"regexp"
	"sort"
	"strconv"
	"strings"
	"time"

	"github.com/Unknwon/goconfig"

	"github.com/pingo"
)

var alipayGatewayNew = `https://mapi.alipay.com/gateway.do?`

type Config struct {
	Partner string
	Key     string
}

type Request struct {
	Service     string
	PaymentType string
	NotifyUrl   string
	ReturnUrl   string
	OutTradeNo  string
	Subject     string
	TotalFee    float64
	Body        string
	ShowUrl     string
	SellerEmail string
}

type Response struct {
	BuyerEmail  string
	OutTradeNo  string
	TradeStatus string
	Subject     string
	TotalFee    float64
}

type Alipay struct{}

func main() {
	plugin := &Alipay{}
	pingo.Register(plugin)
	pingo.Run()

}

var (
	scheme         = "" // 1=https://   0=http://
	encryption_key = "" //取本地app.conf文件中的enkey
	SSLs           = ""
)

/*
入口函数
*/
func (p *Alipay) Action(mapPublic map[string]string, msg *string) error {

	result := ""

	//将老的数据备份一份在本地
	old_mapPublic := make(map[string]string)
	for k, v := range mapPublic {
		old_mapPublic[k] = v
	}

	//获取信息生成sign
	orderid := mapPublic["orderid"]
	amount := mapPublic["amount"]
	pmid := mapPublic["pmid"]

	//初始化一些值 防止为空
	if !IsNum(orderid) {
		orderid = time.Now().Format("20060102150405")
	}
	if !IsNum(amount) {
		amount = "-1"
	}

	if !IsNum(pmid) {
		pmid = GetRandomInt(12)
	}

	rstack_domain := mapPublic["rstack_domain"]
	rstack_ssl := mapPublic["rstack_ssl"]

	scheme = "http://"
	if rstack_ssl == "1" {
		scheme = "https://"
	}

	SSLs = rstack_ssl
	if SSLs != "0" && SSLs != "1" {
		SSLs = "0"
	}

	action := mapPublic["action"]

	if action == "get_url" {
		//fmt.Println("定位1")
		/*
			按规定格式 获得支付宝请求GET URL并返回
		*/
		resellerMode := IF(!IsNum(mapPublic["resellerMode"]), "0", mapPublic["resellerMode"])
		resellerID := IF(!IsNum(mapPublic["resellerID"]), "0", mapPublic["resellerID"])

		urlStr1 := scheme + rstack_domain + "/modules/payment/Alipay/mapi.go"
		urlStr2 := scheme + rstack_domain + "/modules/payment/Alipay/mapi.go"

		curl1 := scheme + rstack_domain + mapPublic["rstack_requesturi"]
		//获取模块接口参数
		parses := GetPayParameter(encryption_key, pmid, curl1, resellerMode, resellerID, rstack_domain)
		//fmt.Println("定位2：", parses)
		if strings.HasPrefix(parses, "-1|") == true {
			result = parses
		} else {
			//提取保存的支付宝 ID KEY
			Email := GetValue(parses, "Email")
			PartnerID := GetValue(parses, "PartnerID")
			Key := GetValue(parses, "Key")

			paytype := IF(!IsNum(GetValue(parses, "paytype")), "0", GetValue(parses, "paytype"))

			orderText := resellerMode + "_" + resellerID + "_" + pmid + "_" + orderid + "_" + paytype

			r_0 := Request{
				NotifyUrl:   urlStr1,
				ReturnUrl:   urlStr2,
				OutTradeNo:  orderText,
				SellerEmail: Email,
				Service:     `create_direct_pay_by_user`,
				PaymentType: `1`,
				Subject:     `支付宝即时交易[` + amount + `]`,
				Body:        `支付宝即时交易[` + amount + `]`,
				TotalFee:    StringToFloat64(amount),
			}

			c := Config{
				Partner: PartnerID,
				Key:     Key,
			}

			orderid = resellerMode + "_" + resellerID + "_" + pmid + "_" + orderid

			result = "0|" + orderid + "|" + ReturnAlipayPage(c, r_0)
		}
	} else if action == "admin_payment_config" || action == "reseller_payment_config" {
		/*
		 支付宝模块默认配置信息项
		*/
		subaction := mapPublic["subaction"]
		pmid := mapPublic["pmid"]
		sign := mapPublic["sign"]

		enSign1 := Md5_default(action + subaction + encryption_key + pmid)

		if enSign1 == sign {
			text2 := Module_OpenFile("Alipay", action)
			if strings.HasPrefix(text2, "-1") == false {
				result = text2
			} else {
				result = "-1|Script file not found!"
			}

		} else {
			result = "-1|Invalid sign|" + pmid
		}

	} else if mapPublic["gaction"] == "showpaytype" {

	} else {

		if mapPublic["rstack_method"] == "post" && mapPublic["notify_id"] != "" {

			result = GetPayResultAsync(mapPublic, encryption_key, rstack_domain)
		} else if mapPublic["notify_id"] != "" {
			result = GetPayResultGet(mapPublic, encryption_key, rstack_domain)
		}
	}

	*msg = result
	return nil
}

func IF(ifstr bool, true1, false2 string) string {
	if ifstr {
		return true1
	}
	return false2
}

//异步
func GetPayResultAsync(map1 map[string]string, encryption_key string, rstack_domain string) string {
	result := ""

	resellerID := IF(!IsNum(map1["resellerID"]), "0", map1["resellerID"])
	resellerMode := IF(!IsNum(map1["resellerMode"]), "0", map1["resellerMode"])

	if len(map1) > 0 {

		trade_status := map1["trade_status"]

		if trade_status == "TRADE_FINISHED" || trade_status == "TRADE_SUCCESS" {
			trade_no := map1["trade_no"]         //交易流水号
			total_fee := map1["total_fee"]       //充值金额返回
			buyer_email := map1["buyer_email"]   //卖家支付宝账号
			out_trade_no := map1["out_trade_no"] //商户订单号 非支付宝号

			notify_id := map1["notify_id"] //通知ID

			description := "支付宝[" + buyer_email + "]充值(" + total_fee + "],交易号:" + trade_no

			if out_trade_no != "" {
				arr0 := strings.Split(out_trade_no, "_")

				if len(arr0) >= 4 {
					//从财务交互取合作者ID+KEY
					mapPost := make(map[string]string)
					mapPost["orderid"] = arr0[3]
					mapPost["action"] = "get_module_parameter"
					mapPost["type"] = "payment"
					mapPost["pmid"] = arr0[2]
					mapPost["sign"] = Md5_default(arr0[2] + arr0[3] + encryption_key)
					cstr, err := ModulePostData(scheme+rstack_domain+"/process?c=modules", mapPost, 5)
					if err != nil {
						return "-1|get_module_parameter Error:" + fmt.Sprint(err)
					}

					if cstr != "" && strings.HasPrefix(cstr, "-1|") == false {
						config2 := Config{
							Partner: GetValue(cstr, "PartnerID"), //合作ID
							Key:     GetValue(cstr, "Key"),       //合作KEY
						}

						//删除多余自带RSTACK字段，保留原始form表单。
						for k, _ := range map1 {
							if strings.HasPrefix(k, "rstack_") == true {
								delete(map1, k)
							}
						}

						//验证表单完整性 防止非法修改
						bool_0 := VerifySign2(config2, map1)

						if bool_0 == true {

							//验证通知ID和支付宝官方效验核对
							bool_1 := VerifyResponse(notify_id, GetValue(cstr, "PartnerID"))

							if bool_1 == "true" {

								//成功，通知财务入款
								mapForm := make(map[string]string)
								mapForm["apiDomain"] = ""
								mapForm["action"] = "complete"
								mapForm["type"] = "payment"
								mapForm["pmid"] = arr0[2]
								mapForm["orderid"] = arr0[3]
								mapForm["resellerMode"] = resellerMode
								mapForm["resellerID"] = resellerID
								mapForm["description"] = description
								mapForm["sign"] = Md5_default(arr0[3] + arr0[2] + encryption_key)
								cstr2, err := ModulePostData(scheme+rstack_domain+"/process?c=modules", mapForm, 10)

								if err == nil {
									if strings.HasPrefix(cstr2, "0") {
										result = "return=success"
									}
								}

							} else {
								result = "return=fail"
							}
						} else {
							result = "return=fail"
						}
					}

				}
			}
		}

	}

	return result
}

//GET返回
func GetPayResultGet(map1 map[string]string, encryption_key string, rstack_domain string) string {
	result := ""

	resellerID := IF(!IsNum(map1["resellerID"]), "0", map1["resellerID"])
	resellerMode := IF(!IsNum(map1["resellerMode"]), "0", map1["resellerMode"])

	if len(map1) > 0 {

		trade_status := map1["trade_status"]

		if trade_status == "TRADE_FINISHED" || trade_status == "TRADE_SUCCESS" {
			trade_no := map1["trade_no"]         //交易流水号
			total_fee := map1["total_fee"]       //充值金额返回
			buyer_email := map1["buyer_email"]   //卖家支付宝账号
			out_trade_no := map1["out_trade_no"] //商户订单号 非支付宝号

			notify_id := map1["notify_id"] //通知ID

			description := "支付宝[" + buyer_email + "]充值(" + total_fee + "],交易号:" + trade_no

			if out_trade_no != "" {
				arr0 := strings.Split(out_trade_no, "_")

				if len(arr0) >= 4 {
					mapPost := make(map[string]string)
					mapPost["orderid"] = arr0[3]
					mapPost["action"] = "get_module_parameter"
					mapPost["type"] = "payment"
					mapPost["pmid"] = arr0[2]
					mapPost["sign"] = Md5_default(arr0[2] + arr0[3] + encryption_key)
					cstr, err := ModulePostData(scheme+rstack_domain+"/process?c=modules", mapPost, 5)
					if err != nil {
						return "-1|get_module_parameter Error:" + fmt.Sprint(err)
					}

					if cstr != "" && strings.HasPrefix(cstr, "-1|") == false {
						config2 := Config{
							Partner: GetValue(cstr, "PartnerID"), //合作ID
							Key:     GetValue(cstr, "Key"),       //合作KEY
						}

						//删除多余自带RSTACK字段，保留原始form表单。
						for k, _ := range map1 {
							if strings.HasPrefix(k, "rstack_") == true {
								delete(map1, k)
							}
						}

						bool_0 := VerifySign2(config2, map1)

						if bool_0 == true {
							bool_1 := VerifyResponse(notify_id, GetValue(cstr, "PartnerID"))

							if bool_1 == "true" {
								mapForm := make(map[string]string)
								mapForm["apiDomain"] = ""
								mapForm["action"] = "complete"
								mapForm["type"] = "payment"
								mapForm["pmid"] = arr0[2]
								mapForm["orderid"] = arr0[3]
								mapForm["resellerMode"] = resellerMode
								mapForm["resellerID"] = resellerID
								mapForm["description"] = description
								mapForm["sign"] = Md5_default(arr0[3] + arr0[2] + encryption_key)
								cstr2, err := ModulePostData(scheme+rstack_domain+"/process?c=modules", mapForm, 10)

								if err == nil {
									if strings.HasPrefix(cstr2, "0") {
										result = "return=" + GetShowAlert("真棒！您成功充值，金额["+total_fee+"],交易号["+trade_no+"]")

									} else {
										result = "return=" + GetShowAlert("未知错误 "+cstr2)
									}
								} else {
									result = "return=" + GetShowAlert("抱歉,模块入款发生错误,Error:"+cstr2)
								}
							} else {
								result = "return=" + GetShowAlert("抱歉,验证不通过,请勿非法提交！")
							}
						} else {
							result = "return=" + GetShowAlert("抱歉,表单不一致,请勿非法提交！")
						}

					} else {
						result = "return=" + GetShowAlert("抱歉,请求POST获取模块参数错误,错误类型:get_module_parameter,错误信息："+cstr)
					}
				} else {
					result = "return=" + GetShowAlert("抱歉,商户订单号格式不正确！")
				}
			} else {
				result = "return=" + GetShowAlert("抱歉,返回商户订单号为空！")
			}
		} else {
			result = "return=" + GetShowAlert("抱歉,付款还未完成！交易状态码:"+trade_status)
		}
	} else {
		result = "return=" + GetShowAlert("抱歉,无返回参数，请求来源地，重新发起请求！")
	}
	return result
}

func GetShowAlert(str string) string {
	return "<script type=\"text/javascript\"> alert('" + str + "');window.opener= null;window.open('','_self'); window.close();</script>"
}

func GetPayParameter(encryption_key string, pmid string, currentUrl string, resellerMode string, resellerID string, rHost string) string {

	//curl := currentUrl

	//去尾巴
	if currentUrl != "" {
		currentUrl = currentUrl[0 : strings.LastIndex(currentUrl, "/")+1]
	}

	map1 := make(map[string]string)
	map1["apiDomain"] = ""
	map1["action"] = "get_parameter" //获取模块config字段的JSON自定义参数。
	map1["type"] = "payment"
	map1["resellerMode"] = resellerMode
	map1["resellerID"] = resellerID
	map1["pmid"] = pmid
	map1["sign"] = Md5_default(pmid + encryption_key + currentUrl)

	// scheme = "http://"
	// rHost = rHost + ":8888"
	cstr, err := ModulePostData(scheme+rHost+"/process?c=modules", map1, 10)
	if err != nil {
		return "-1|get_parameter Error:" + fmt.Sprint(err)
	}

	return cstr

}

//模块提交POST交互
func ModulePostData(urladdr string, map1 map[string]string, timeout int) (ret string, err error) {

	var client *http.Client
	if strings.HasPrefix(urladdr, "https://") == true {
		tr := &http.Transport{
			TLSClientConfig: &tls.Config{InsecureSkipVerify: true},
		}
		client = &http.Client{Transport: tr}
	} else {
		client = &http.Client{}
	}
	//client := &http.Client{Transport: tr}
	//client := &http.Client{}

	fields := ""
	for k, v := range map1 {
		if fields == "" {
			fields = k + "=" + v
		} else {
			fields += "&" + k + "=" + v
		}
	}

	req, err := http.NewRequest("POST", urladdr, strings.NewReader(fields))
	if err != nil {
		return "", err
	}

	req.Header.Add("Content-Type", "application/x-www-form-urlencoded")
	req.Header.Add("Accept", "*/*")
	req.Header.Add("User-Agent", "RStack Platform/1.0")

	resp, err := client.Do(req)
	if err != nil {
		return "", err
	}

	defer resp.Body.Close()

	body, err := ioutil.ReadAll(resp.Body)

	if err != nil {
		return "", err
	}

	return string(body), nil

}

//打开文件错误返回 -1|错误信息
func Module_OpenFile(moduleName string, str string) string {
	result := ""
	var err error
	if str != "" {
		result, err = OpenFile("static/modules/payment/" + moduleName + "/js/" + str + ".js")
		if err != nil {
			result = "-1|Script file not found"
			return result
		}
	}

	return result

}

func GetRandomInt(l int) string {
	str := "0123456789"
	bytes := []byte(str)
	result := []byte{}

	r := rand.New(rand.NewSource(time.Now().UnixNano()))

	for i := 0; i < l; i++ {
		result = append(result, bytes[r.Intn(len(bytes))])
	}
	time.Sleep(1) //间隔  否则会出现重复随机数
	return string(result)
}

func OpenFile(path string) (str string, err error) {
	f, err := os.Open(path)
	if err != nil {
		return "", err
	}

	str0, err := ioutil.ReadAll(f)

	if err != nil {
		return "", err
	}

	str = string(str0)

	return str, nil
}

type kvpair struct {
	k, v string
}

type kvpairs []kvpair

func (t kvpairs) Less(i, j int) bool {
	return t[i].k < t[j].k
}

func (t kvpairs) Swap(i, j int) {
	t[i], t[j] = t[j], t[i]
}

func (t kvpairs) Len() int {
	return len(t)
}

func (t kvpairs) Sort() {
	sort.Sort(t)
}

func (t kvpairs) RemoveEmpty() (t2 kvpairs) {
	for _, kv := range t {
		if kv.v != "" {
			t2 = append(t2, kv)
		}
	}
	return
}

func (t kvpairs) Join() string {
	var strs []string
	for _, kv := range t {
		strs = append(strs, kv.k+"="+kv.v)
	}
	return strings.Join(strs, "&")
}

func md5Sign(str, key string) string {
	h := md5.New()
	io.WriteString(h, str)
	io.WriteString(h, key)
	return fmt.Sprintf("%x", h.Sum(nil))
}

func Md5_Diy(md5_string string) string {
	h := md5.New()
	h.Write([]byte(md5_string + "Cissv.Com")) //xsNode.Com@#&##
	return hex.EncodeToString(h.Sum(nil))
}

func VerifyResponse(notify_id string, alipay_partner string) string {
	var HTTPS_VERIFY_URL = "https://mapi.alipay.com/gateway.do?service=notify_verify&"
	hezuoShenfen := alipay_partner //合作ID
	veryfy_url := HTTPS_VERIFY_URL + "partner=" + hezuoShenfen + "&notify_id=" + notify_id
	resp, err := http.Post(veryfy_url, "text/plain;charset=UTF-8", nil)
	if err != nil {
		//fmt.Println(err)
		return "false"
	}

	body, err := ioutil.ReadAll(resp.Body)

	if err != nil {
		//fmt.Println(err)
		return "false"
	}
	defer resp.Body.Close()

	return string(body)
}

var HTTPS_VERIFY_URL = "https://mapi.alipay.com/gateway.do?service=notify_verify&"

func VerifySign(c Config, u url.Values) (bool_0 bool) {

	p := kvpairs{}
	sign := ""
	for k := range u {
		v := u.Get(k)
		switch k {
		case "sign":
			sign = v
			continue
		case "sign_type":
			continue
		}
		p = append(p, kvpair{k, v})
	}
	if sign == "" {
		//err = fmt.Errorf("sign not found")
		return false
	}
	p = p.RemoveEmpty()
	p.Sort()

	if md5Sign(p.Join(), c.Key) != sign {

		return false
	}

	//veryfy_url := HTTPS_VERIFY_URL + "partner=" + alipayConfig.Partner + "&notify_id=" + u.Get("notify_id")

	return true
}

func VerifySign2(c Config, map1 map[string]string) (bool_0 bool) {
	p := kvpairs{}
	sign := ""
	for k, v := range map1 {
		switch k {
		case "sign":
			sign = v
			continue
		case "sign_type":
			continue
		}
		p = append(p, kvpair{k, v})
	}
	if sign == "" {
		return false
	}
	p = p.RemoveEmpty()
	p.Sort()

	if md5Sign(p.Join(), c.Key) != sign {
		return false
	}

	return true

}

func ParseResponse(c Config, p url.Values) (r Response, err error) {

	res := VerifySign(c, p)

	if res == false {
		return
	}

	r.BuyerEmail = p.Get("buyer_email")
	r.TradeStatus = p.Get("trade_status")
	r.OutTradeNo = p.Get("out_trade_no")
	r.Subject = p.Get("subject")
	fmt.Sscanf(p.Get("total_fee"), "%f", &r.TotalFee)

	if r.TradeStatus != "TRADE_SUCCESS" && r.TradeStatus != "TRADE_FINISHED" {
		err = fmt.Errorf("trade not success or finnished")
		return
	}
	return
}

func StringToFloat64(str string) float64 {
	v1, err := strconv.ParseFloat(str, 64)
	if err != nil {
		return 0
	}

	return v1
}

func IsNum(isnumber string) bool {

	b2, err := regexp.MatchString("^[+-]?\\d*[.]?\\d*$", isnumber)
	if err != nil {
		return false
	}
	return b2 && isnumber != ""
}

func GetValue(json, string_0 string) string {
	text := ""

	string_0 = "\"" + string_0 + "\":" //也就是 如 "pid":

	if strings.Index(json, string_0) > -1 {
		//"9","name":"2G产品2048内存","remark":"&lt;
		text = json[strings.Index(json, string_0)+len(string_0):]
		//fmt.Println("第一：", text)
		num := 0
		num2 := 0

		if strings.HasPrefix(text, "\"") == true {

			//如果前面  "开头的 代表字符串JSON内容
			if strings.Index(text, "\",\"") > -1 {

				//如果找到 ","
				num2 = strings.Index(text, "\",\"")

			} else {
				num2 = strings.LastIndex(text, "\"")
			}
			num = 1
			//num2-- //如果加了这句 会获取不到 或则缺少东西
		} else if strings.HasPrefix(text, "{") == true {

			//如果是内置JSON到JSON
			if strings.Index(text, "\"},\"") > 0 {

				//找到 "},"
				num2 = strings.Index(text, "\"},\"") + 2 //   "},"
			} else if strings.Index(text, "\"}") > 0 { // "}

				num2 = strings.Index(text, "\"}") + 2
			} else if strings.Index(text, "}") > 0 { // }

				num2 = strings.Index(text, "}") + 1
			}

		} else if strings.Index(text, ",") > -1 {
			num2 = strings.Index(text, ",")
		} else {

			num2 = strings.Index(text, "}")
		}

		if num > num2 {
			//防止索引错误导致程序异常。
			text = ""
		} else {
			text = text[num:num2]
		}

		if strings.HasPrefix(text, "{") == false && strings.Index(text, "\"") > -1 {
			text = text[0:strings.Index(text, "\"")]

		}
	}

	return text
}

func Md5_default(md5_default string) string {
	h := md5.New()
	h.Write([]byte(md5_default))
	return hex.EncodeToString(h.Sum(nil))
}

//c Config, r Request
func ReturnAlipayPage(c Config, r Request) string {
	// var c Config
	// var r Request

	p := kvpairs{
		kvpair{`_input_charset`, `utf-8`},
		kvpair{`out_trade_no`, r.OutTradeNo},
		kvpair{`partner`, c.Partner},
		kvpair{`payment_type`, r.PaymentType},
		kvpair{`notify_url`, r.NotifyUrl},
		kvpair{`return_url`, r.ReturnUrl},
		kvpair{`subject`, r.Subject},
		kvpair{`total_fee`, fmt.Sprintf("%.2f", r.TotalFee)},
		kvpair{`body`, r.Body},
		kvpair{`service`, r.Service},
		kvpair{`show_url`, r.ShowUrl},
		kvpair{`seller_email`, r.SellerEmail},
	}
	p = p.RemoveEmpty()
	p.Sort()
	sign := md5Sign(p.Join(), c.Key)
	p = append(p, kvpair{`sign`, sign})
	p = append(p, kvpair{`sign_type`, `MD5`})

	adds := ""
	httpUrl := alipayGatewayNew

	for _, value := range p {
		if adds == "" {
			adds = value.k + "=" + value.v
		} else {
			adds += "&" + value.k + "=" + url.QueryEscape(value.v)
		}
	}

	httpUrl += adds

	return httpUrl

}

func NewPage(c Config, r Request, w io.Writer) {
	p := kvpairs{
		kvpair{`_input_charset`, `utf-8`},
		kvpair{`out_trade_no`, r.OutTradeNo},
		kvpair{`partner`, c.Partner},
		kvpair{`payment_type`, r.PaymentType},
		kvpair{`notify_url`, r.NotifyUrl},
		kvpair{`return_url`, r.ReturnUrl},
		kvpair{`subject`, r.Subject},
		kvpair{`total_fee`, fmt.Sprintf("%.2f", r.TotalFee)},
		kvpair{`body`, r.Body},
		kvpair{`service`, r.Service},
		kvpair{`show_url`, r.ShowUrl},
		kvpair{`seller_email`, r.SellerEmail},
	}
	p = p.RemoveEmpty()
	p.Sort()

	sign := md5Sign(p.Join(), c.Key)
	p = append(p, kvpair{`sign`, sign})
	p = append(p, kvpair{`sign_type`, `MD5`})
	//	for _, kv := range p {
	//		fmt.Println("新K等于：", kv.k, " 新V等于：", kv.v)
	//	}

	fmt.Fprintln(w, `<html><head>
	<meta http-equiv="Content-Type" content="text/html; charset=utf-8">
	</head><body>`)
	fmt.Fprintf(w, `<form name='alipaysubmit' action='%s_input_charset=utf-8' method='post'> `, alipayGatewayNew)
	for _, kv := range p {
		fmt.Fprintf(w, `<input type='hidden' name='%s' value='%s' />`, kv.k, kv.v)
	}
	fmt.Fprintln(w, `<script>document.forms['alipaysubmit'].submit();</script>`)
	fmt.Fprintln(w, `</body></html>`)
}

func init() {

	pwd, err := os.Getwd()
	if err != nil {

		log.Println("Get Pwd Error：", err)
		return
	} else {
		pwd = strings.Replace(pwd, "\\", "/", -1)
		pwd = strings.Replace(pwd, "/bin", "", -1)

		//fmt.Println("路径：", pwd)
		cfg, err := goconfig.LoadConfigFile(pwd + "/conf/app.conf")
		if err != nil {
			log.Println("Load app.conf Error:", err)
		} else {

			enKey, err := cfg.GetValue("listen", "enKey")
			if err != nil {
				log.Println("Get listen domain", err)
				return
			}
			encryption_key = enKey

			//如果未设置，随机生成一个字符串enKey，让API失效。
			if encryption_key == "" {
				encryption_key = time.Now().Format("20060102150405")
			}
		}
	}

}
